"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = _default;

var _fs = _interopRequireDefault(require("fs"));

var _jsonStreamStringify = _interopRequireDefault(require("json-stream-stringify"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

function _construct(Parent, args, Class) { if (isNativeReflectConstruct()) { _construct = Reflect.construct; } else { _construct = function _construct(Parent, args, Class) { var a = [null]; a.push.apply(a, args); var Constructor = Function.bind.apply(Parent, a); var instance = new Constructor(); if (Class) _setPrototypeOf(instance, Class.prototype); return instance; }; } return _construct.apply(null, arguments); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

var JSONStringify = function JSONStringify() {
  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  return new Promise(function (resolve, reject) {
    var stream = _construct(_jsonStreamStringify["default"], args);

    stream.once('error', function () {
      try {
        var results = JSON.stringify.apply(JSON, args);
        resolve(results);
      } catch (error) {
        reject(error);
      }
    });
    var trunks = '';
    stream.on('data', function (trunk) {
      trunks += trunk;
    });
    stream.on('end', function () {
      resolve(trunks);
    });
  });
};

function _default() {
  for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    args[_key2] = arguments[_key2];
  }

  var pluginOptions = args[1];
  var options = Object.assign({
    extractedFile: './src/translations/aggregated.json',
    langFiles: [{
      path: './src/translations/ru.json',
      cleanUpNewMessages: false
    }, {
      path: './src/translations/en.json',
      cleanUpNewMessages: true
    }]
  }, pluginOptions);
  /**
   * @returns {Array}
   */

  function getTargetFileContent() {
    var doesFileExists = _fs["default"].existsSync(options.extractedFile);

    if (!doesFileExists) {
      return [];
    }

    var result = JSON.parse(_fs["default"].readFileSync(options.extractedFile));
    return Array.isArray(result) ? result : [];
  }
  /**
   * @returns {Object.<string, string>}
   */


  function getTargetLangFileContent(path) {
    var doesFileExists = _fs["default"].existsSync(path);

    if (!doesFileExists) {
      return {};
    }

    return Object.assign({}, JSON.parse(_fs["default"].readFileSync(path)));
  }
  /**
   * @param {{id: string}[]} existing
   * @param {{id: string}[]} descriptors
   *
   * @returns {{id: string}[]}
   */


  function mergeDescriptors(existing, descriptors) {
    return existing // Remove from existing all messages with id, that used in descriptors
    .filter(function (descriptor) {
      return descriptors.findIndex(function (_ref) {
        var id = _ref.id;
        return id === descriptor.id;
      }) === -1;
    }) // Merge with new descriptors array
    .concat(descriptors).sort(function (a, b) {
      return a.id < b.id ? -1 : a.id > b.id ? 1 : 0;
    });
  }

  function mergeMessages(cleanUpNewMessages, oldMessages, newMessages) {
    if (!cleanUpNewMessages) {
      return Object.assign(oldMessages, newMessages);
    }

    var result = Object.assign({}, oldMessages);
    Object.keys(newMessages).forEach(function (id) {
      if (!result[id]) {
        result[id] = '';
      }
    });
    return result;
  }

  return {
    post: function post(file) {
      return new Promise(function (resolve) {
        var _file$metadata$react, _file$metadata$react$;

        if (!((_file$metadata$react = file.metadata['react-intl']) === null || _file$metadata$react === void 0 ? void 0 : (_file$metadata$react$ = _file$metadata$react.messages) === null || _file$metadata$react$ === void 0 ? void 0 : _file$metadata$react$.length)) {
          return;
        }

        var descriptors = file.metadata['react-intl'].messages;

        if (!Array.isArray(descriptors)) {
          return;
        }

        var existingDescriptors = getTargetFileContent();
        var resultDescriptors = mergeDescriptors(existingDescriptors, descriptors);

        if (!(resultDescriptors === null || resultDescriptors === void 0 ? void 0 : resultDescriptors.length)) {
          return;
        }

        JSONStringify(resultDescriptors, null, 2).then(function (jsonStr) {
          _fs["default"].writeFileSync(options.extractedFile, jsonStr);

          var messages = {};
          resultDescriptors.forEach(function (_ref2) {
            var id = _ref2.id,
                defaultMessage = _ref2.defaultMessage;
            messages[id] = defaultMessage || '';
          });
          var promises = options.langFiles.map(function (_ref3) {
            var path = _ref3.path,
                cleanUpNewMessages = _ref3.cleanUpNewMessages;
            return new Promise(function (resolve) {
              var resultMessages = mergeMessages(cleanUpNewMessages, getTargetLangFileContent(path), messages);
              JSONStringify(resultMessages, null, 2).then(function (jsonStr) {
                _fs["default"].writeFileSync(path, jsonStr);

                resolve();
              });
            });
          });
          Promise.all(promises).then(function () {
            resolve();
          });
        });
      });
    }
  };
}